@@ -713,6 +713,9 @@ class ApiField:
     """"""
     NN_UPDATED = "nnUpdated"
     """"""
+    M_GUIDE_ID = (["meta", "guide"], "guide_id")
+    """"""
+    GUIDE_ID = "guideId"
 
 
 def _get_single_item(items):
