@@ -7,7 +7,6 @@ from supervisely.app.widgets.checkbox.checkbox import Checkbox
 from supervisely.app.widgets.container.container import Container
 from supervisely.app.widgets.field.field import Field
 from supervisely.app.widgets.select.select import Select
-from supervisely.app.widgets.select_project.select_project import SelectProject
 from supervisely.app.widgets.tree_select.tree_select import TreeSelect
 from supervisely.project.project_type import ProjectType
 
@@ -120,7 +119,15 @@ class SelectDatasetTree(Widget):
         if self._project_id:
             project_info = self._api.project.get_info_by_id(self._project_id)
             if allowed_project_types is not None:
-                if project_info.type not in [pt.value for pt in allowed_project_types]:
+                allowed_values = []
+                if not isinstance(allowed_project_types, list):
+                    allowed_project_types = [allowed_project_types]
+
+                for pt in allowed_project_types:
+                    if isinstance(pt, (ProjectType, str)):
+                        allowed_values.append(str(pt))
+
+                if project_info.type not in allowed_values:
                     self._project_id = None
 
         self._multiselect = multiselect
@@ -322,6 +329,7 @@ class SelectDatasetTree(Widget):
         """
         if not self._multiselect:
             raise ValueError("This method can only be called when multiselect is enabled.")
+        self._select_all_datasets_checkbox.uncheck()
         self._select_dataset.set_selected_by_id(dataset_ids)
 
     def team_changed(self, func: Callable) -> Callable:
