@@ -68,6 +68,7 @@ class BaseTrainArtifacts:
         self._pattern: str = None
         self._available_task_types: List[str] = []
         self._require_runtime = False
+        self._has_benchmark_evaluation = False
 
     @property
     def team_id(self) -> int:
@@ -209,6 +210,13 @@ class BaseTrainArtifacts:
         """
         return self._require_runtime
 
+    @property
+    def has_benchmark_evaluation(self):
+        """
+        Whether the framework has integrated benchmark evaluation.
+        """
+        return self._has_benchmark_evaluation
+
     def is_valid_artifacts_path(self, path):
         """
         Check if the provided path is valid and follows specified session path pattern.
@@ -610,9 +618,9 @@ class BaseTrainArtifacts:
             date_time = parsed_datetime.strftime("%Y-%m-%d %H:%M:%S")
 
             experiment_info_data = {
-                "experiment_name": f"Unknown {self.framework_name} experiment",
+                "experiment_name": f"{self.framework_name} experiment",
                 "framework_name": self.framework_name,
-                "model_name": f"Unknown {self.framework_name} model",
+                "model_name": f"{self.framework_name} model",
                 "task_type": train_info.task_type,
                 "project_id": project_id,
                 "task_id": train_info.task_id,
