@@ -180,7 +180,13 @@ class TrainValSplitsSelector:
                 return False
 
             # Check if datasets are not empty
-            filters = [{ ApiField.FIELD: ApiField.ID, ApiField.OPERATOR: "in", ApiField.VALUE: train_dataset_id + val_dataset_id}]
+            filters = [
+                {
+                    ApiField.FIELD: ApiField.ID,
+                    ApiField.OPERATOR: "in",
+                    ApiField.VALUE: train_dataset_id + val_dataset_id,
+                }
+            ]
             selected_datasets = self.api.dataset.get_list(self.project_id, filters, recursive=True)
             datasets_count = {}
             for dataset in selected_datasets:
@@ -334,6 +340,7 @@ class TrainValSplitsSelector:
 
     def _detect_splits(self, collections_split: bool, datasets_split: bool) -> bool:
         """Detect splits based on the selected method"""
+        self._parse_collections()
         splits_found = False
         if collections_split:
             splits_found = self._detect_collections()
@@ -341,47 +348,59 @@ class TrainValSplitsSelector:
             splits_found = self._detect_datasets()
         return splits_found
 
+    def _parse_collections(self) -> None:
+        """Parse collections with train and val prefixes and set them to train_val_splits variables"""
+        all_collections = self.api.entities_collection.get_list(self.project_id)
+        existing_train_collections = [
+            collection for collection in all_collections if collection.name.startswith("train_")
+        ]
+        existing_val_collections = [
+            collection for collection in all_collections if collection.name.startswith("val_")
+        ]
+
+        self._all_train_collections = existing_train_collections
+        self._all_val_collections = existing_val_collections
+        self._latest_train_collection = self._get_latest_collection(existing_train_collections, "train")
+        self._latest_val_collection = self._get_latest_collection(existing_val_collections, "val")
+
+    def _get_latest_collection(
+        self, collections: List[EntitiesCollectionInfo], expected_prefix: str
+    ) -> EntitiesCollectionInfo:
+        curr_collection = None
+        curr_idx = 0
+        for collection in collections:
+            parts = collection.name.split("_")
+            if len(parts) == 2:
+                prefix = parts[0].lower()
+                if prefix == expected_prefix:
+                    if parts[1].isdigit():
+                        collection_idx = int(parts[1])
+                        if collection_idx > curr_idx:
+                            curr_idx = collection_idx
+                            curr_collection = collection
+        return curr_collection
+
+
     def _detect_collections(self) -> bool:
         """Find collections with train and val prefixes and set them to train_val_splits"""
-        def _get_latest_collection(collections: List[EntitiesCollectionInfo]) -> EntitiesCollectionInfo:
-            curr_collection = None
-            curr_idx = 0
-            for collection in collections:
-                collection_idx = int(collection.name.rsplit('_', 1)[-1])
-                if collection_idx > curr_idx:
-                    curr_idx = collection_idx
-                    curr_collection = collection
-            return curr_collection
 
-        all_collections = self.api.entities_collection.get_list(self.project_id)
-        train_collections = []
-        val_collections = []
         collections_found = False
-        for collection in all_collections:
-            if collection.name.lower().startswith("train_"):
-                train_collections.append(collection)
-            elif collection.name.lower().startswith("val_"):
-                val_collections.append(collection)
-
-        train_collection = _get_latest_collection(train_collections)
-        val_collection = _get_latest_collection(val_collections)
-        if train_collection is not None and val_collection is not None:
-            self.train_val_splits.set_collections_splits([train_collection.id], [val_collection.id])
+        if self._latest_train_collection is not None and self._latest_val_collection is not None:
+            self.train_val_splits.set_collections_splits(
+                [self._latest_train_collection.id], [self._latest_val_collection.id]
+            )
             self.validator_text = Text("Train and val collections are detected", status="info")
             self.validator_text.show()
             collections_found = True
-            self._all_train_collections = train_collections
-            self._all_val_collections = val_collections
-            self._latest_train_collection = train_collection
-            self._latest_val_collection = val_collection
         else:
             self.validator_text = Text("")
             self.validator_text.hide()
             collections_found = False
         return collections_found
-        
+
     def _detect_datasets(self) -> bool:
         """Find datasets with train and val prefixes and set them to train_val_splits"""
+
         def _extend_with_nested(root_ds):
             nested = self.api.dataset.get_nested(self.project_id, root_ds.id)
             nested_ids = [ds.id for ds in nested]
@@ -407,7 +426,9 @@ class TrainValSplitsSelector:
         val_count = len(train_val_dataset_ids["val"])
 
         if train_count > 0 and val_count > 0:
-            self.train_val_splits.set_datasets_splits(train_val_dataset_ids["train"], train_val_dataset_ids["val"])
+            self.train_val_splits.set_datasets_splits(
+                train_val_dataset_ids["train"], train_val_dataset_ids["val"]
+            )
             datasets_found = True
 
         if train_count > 0 and val_count > 0:
@@ -415,7 +436,7 @@ class TrainValSplitsSelector:
                 message = "train and val datasets are detected"
             else:
                 message = "Multiple train and val datasets are detected. Check manually if selection is correct"
-            
+
             self.validator_text = Text(message, status="info")
             self.validator_text.show()
             datasets_found = True
@@ -423,4 +444,4 @@ class TrainValSplitsSelector:
             self.validator_text = Text("")
             self.validator_text.hide()
             datasets_found = False
-        return datasets_found
\ No newline at end of file
+        return datasets_found
