@@ -1193,7 +1193,7 @@ class VideoProject(Project):
         api: Api,
         project_id: int,
         dest_dir: str,
-        semaphore: asyncio.Semaphore = None,
+        semaphore: Optional[Union[asyncio.Semaphore, int]] = None,
         dataset_ids: List[int] = None,
         download_videos: bool = True,
         save_video_info: bool = False,
@@ -1213,7 +1213,7 @@ class VideoProject(Project):
         :param dest_dir: Directory to download video project.
         :type dest_dir: :class:`str`
         :param semaphore: Semaphore to limit the number of concurrent downloads of items.
-        :type semaphore: :class:`asyncio.Semaphore`, optional
+        :type semaphore: :class:`asyncio.Semaphore` or :class:`int`, optional
         :param dataset_ids: Datasets IDs in Supervisely to download.
         :type dataset_ids: :class:`list` [ :class:`int` ], optional
         :param download_videos: Download videos from Supervisely video project in dest_dir or not.
@@ -1584,7 +1584,7 @@ async def download_video_project_async(
     api: Api,
     project_id: int,
     dest_dir: str,
-    semaphore: Optional[asyncio.Semaphore] = None,
+    semaphore: Optional[Union[asyncio.Semaphore, int]] = None,
     dataset_ids: Optional[List[int]] = None,
     download_videos: Optional[bool] = True,
     save_video_info: Optional[bool] = False,
@@ -1604,7 +1604,7 @@ async def download_video_project_async(
     :param dest_dir: Destination path to local directory.
     :type dest_dir: str
     :param semaphore: Semaphore to limit the number of simultaneous downloads of items.
-    :type semaphore: asyncio.Semaphore, optional
+    :type semaphore: asyncio.Semaphore or int, optional
     :param dataset_ids: Specified list of Dataset IDs which will be downloaded. Datasets could be downloaded from different projects but with the same data type.
     :type dataset_ids: list(int), optional
     :param download_videos: Include videos in the download.
@@ -1643,6 +1643,8 @@ async def download_video_project_async(
     """
     if semaphore is None:
         semaphore = api.get_default_semaphore()
+    elif isinstance(semaphore, int):
+        semaphore = asyncio.Semaphore(semaphore)
 
     key_id_map = KeyIdMap()
 
@@ -1734,7 +1736,7 @@ def _log_warning(
 async def _download_project_item_async(
     api: Api,
     video: VideoInfo,
-    semaphore: asyncio.Semaphore,
+    semaphore: Union[asyncio.Semaphore, int],
     dataset: DatasetInfo,
     dest_dir: str,
     project_fs: Project,
@@ -1749,6 +1751,9 @@ async def _download_project_item_async(
     This function downloads a video item from the project in Supervisely platform asynchronously.
     """
 
+    if isinstance(semaphore, int):
+        semaphore = asyncio.Semaphore(semaphore)
+
     try:
         ann_json = await api.video.annotation.download_async(video.id, video, semaphore=semaphore)
         ann_json = ann_json[0]
