@@ -537,11 +537,9 @@ class VideoFrameReader:
             try:
                 import decord
 
-                self.vr = decord.VideoReader(str(self.video_path))
+                self.vr = decord.VideoReader(str(self.video_path), num_threads=1)
             except ImportError:
-                default_logger.debug(
-                    "Decord is not installed. Falling back to OpenCV for video reading."
-                )
+                default_logger.debug("Decord is not installed. Falling back to OpenCV for video reading.")
                 self.cap = cv2.VideoCapture(str(self.video_path))
 
     def close(self):
@@ -562,24 +560,30 @@ class VideoFrameReader:
     def __del__(self):
         self.close()
 
-    def iterate_frames(self, frame_indexes: List[int] = None) -> Generator[np.ndarray, None, None]:
+    def iterate_frames(self, frame_indexes: Optional[List[int]] = None) -> Generator[np.ndarray, None, None]:
         self._ensure_initialized()
         if frame_indexes is None:
             frame_indexes = self.frame_indexes
         if self.vr is not None:
+            # Decord
             if frame_indexes is None:
                 frame_indexes = range(len(self.vr))
-            for frame_index in frame_indexes:
-                frame = self.vr[frame_index]
-                yield frame.asnumpy()
+            for idx in frame_indexes:
+                arr = self.vr[idx].asnumpy()
+                yield arr
+                del arr
         else:
+            # OpenCV fallback
             if frame_indexes is None:
                 frame_count = int(self.cap.get(cv2.CAP_PROP_FRAME_COUNT))
                 frame_indexes = range(frame_count)
             for frame_index in frame_indexes:
-                if 1 > frame_index - self.prev_idx < 20:
+                if 1 < frame_index - self.prev_idx < 20:
                     while self.prev_idx < frame_index - 1:
-                        self.cap.read()
+                        ok, _ = self.cap.read()
+                        if not ok:
+                            break
+                        self.prev_idx += 1
                 if frame_index != self.prev_idx + 1:
                     self.cap.set(cv2.CAP_PROP_POS_FRAMES, frame_index)
                 ret, frame = self.cap.read()
@@ -588,6 +592,17 @@ class VideoFrameReader:
                 yield cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
                 self.prev_idx = frame_index
 
+    def read_batch(self, frame_indexes: List[int]) -> List[np.ndarray]:
+        self._ensure_initialized()
+        if self.vr is not None:
+            batch_nd = self.vr.get_batch(frame_indexes)
+            batch_np = batch_nd.asnumpy()
+            frames = [batch_np[i].copy() for i in range(batch_np.shape[0])]
+            del batch_np
+            return frames
+        else:
+            return list(self.iterate_frames(frame_indexes))
+
     def read_frames(self, frame_indexes: List[int] = None) -> List[np.ndarray]:
         return list(self.iterate_frames(frame_indexes))
 
